#!/usr/bin/env perl
use strict;
use warnings;

use Test::More tests => 16;
use Test::NoWarnings;
use Data::Dump 'pp';

BEGIN { use_ok('List::Index'); }
ok(eval { List::Index->VERSION(1) }, 'version 1.00 compatibility');

{
my @uniform = 'a'..'z';
ok(my $index = List::Index->new(\@uniform), 'object (single-char values)');
is_deeply(\@uniform, ['a'..'z'], 'original data unaltered');
is_deeply($index->ranges, [['','']], 'single page');
is_deeply($index->ranges({pages => 3}), [map { [split /-/, $_, 3] } qw(
	-h i-q r-
)], 'given pages');
is_deeply($index->ranges({pagesize => @uniform / 2.1}), [map { [split /-/, $_, 2] } qw(
	-h i-q r-
)], 'equivalent pagesize');
}

{
ok(my $index = List::Index->new(['aa'..'zz', 1..193]), 'non-alphabetic values (uniform)');
is_deeply($index->ranges, [map { [split /-/, $_, 2] } qw(
	-. ..-. .. ...-.
	a-bv bw-dr ds-fn fo-hk hl-jg jh-k l-m n-ov ow-qr qs-sn so-uk ul-wg wh-x y-
)], 'default ranges');
is_deeply($index->ranges({pagesize => 300}), [map { [split /-/, $_, 2] } qw(
	-c d-ov ow-
)], 'large pagesize');
}

{
ok(my $index = List::Index->new([qw(
	baa1 baa2  baa3 baaa  bbc cbc  daaa ea  eaaa zed
)]), 'variable length values');
is_deeply($index->ranges({pagesize => 2, context => 0}), [
	map { [split /-/, $_, 2] } qw(-baa baa.-bbb bbc-daa. daaa-eaa. eaaa-)
], 'no context');
is_deeply($index->ranges({pagesize => 2}), [
	map { [split /-/, $_, 2] } qw(-baa baa.-ba bb-c d-ea. eaa-)
], 'default context');  # context should be 1
is_deeply($index->ranges({pagesize => 2, context => 2}), [
	map { [split /-/, $_, 2] } qw(-a b-ba bb-c d e-)
], 'overlap');  # first item equals second due to large context
is_deeply($index->ranges({pagesize => 2, length => 1}), [
	map { [split /-/, $_, 2] } qw(-a b-a b-c d e-)
], 'single char');

#pp($index->ranges({pagesize => 2, context => 2, length => 1}));
}

